"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadToFile = exports.fetchWithTimeout = exports.throwIfNotOk = void 0;
const node_fetch_1 = require("node-fetch");
const util_1 = require("util");
const path_1 = require("path");
const fs_1 = require("fs");
const coded_error_1 = require("@carnesen/coded-error");
const mkdirp = require("mkdirp");
const pump = require("pump");
const rimraf_1 = require("rimraf");
const alwayscli_1 = require("@alwaysai/alwayscli");
const random_string_1 = require("./random-string");
const constants_1 = require("../constants");
const logger_1 = require("./logger");
const stringify_error_1 = require("./stringify-error");
async function throwIfNotOk(response) {
    if (!response.ok) {
        logger_1.logger.error(`Status: ${response.status}, ${response.statusText}`);
        const message = `Server responded ${response.status} ("${response.statusText}")`;
        const code = response.status;
        throw new coded_error_1.CodedError(message, code);
    }
}
exports.throwIfNotOk = throwIfNotOk;
async function writeToFile(path, readable) {
    mkdirp.sync((0, path_1.dirname)(path));
    const tmpFilePath = `${path}.${(0, random_string_1.RandomString)()}.tmp`;
    try {
        await new Promise((resolve, reject) => {
            const writeable = (0, fs_1.createWriteStream)(tmpFilePath);
            pump(readable, writeable, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
        await (0, util_1.promisify)(fs_1.rename)(tmpFilePath, path);
    }
    catch (exception) {
        try {
            await (0, rimraf_1.default)(tmpFilePath);
        }
        catch (err) {
            logger_1.logger.warn('Failed to delete %s! %s', tmpFilePath, (0, stringify_error_1.stringifyError)(err));
        }
        throw exception;
    }
}
async function fetchWithTimeout(url, options = {}) {
    const { timeout = 60000 } = options;
    const controller = new AbortController();
    const id = setTimeout(() => controller.abort(), timeout);
    try {
        const response = await (0, node_fetch_1.default)(url, Object.assign(Object.assign({}, options), { signal: controller.signal }));
        clearTimeout(id);
        await throwIfNotOk(response);
        return response;
    }
    catch (error) {
        clearTimeout(id);
        if (error.name === 'AbortError') {
            throw new Error('Request timed out');
        }
        else if (error instanceof node_fetch_1.FetchError) {
            throw new Error(`Fetch error: ${error.message}`);
        }
        else {
            throw error;
        }
    }
}
exports.fetchWithTimeout = fetchWithTimeout;
async function downloadToFile(props) {
    const { url, path } = props;
    logger_1.logger.debug(`Downloading package from ${url}`);
    const response = await fetchWithTimeout(url);
    if (response.body === null) {
        throw new alwayscli_1.CliTerseError(`Error downloading ${url}. ${constants_1.PLEASE_REPORT_THIS_ERROR_MESSAGE}`);
    }
    await writeToFile(path, response.body);
}
exports.downloadToFile = downloadToFile;
//# sourceMappingURL=download-to-file.js.map